//=================================================================================================================================//
//DESCRIPTION: A tool for silently relinking files in an InDesign document.  
//
// This script was originally written by Jeremy Howard (www.jeremyhoward.me). 
//
// It has been modified by Daniel Reinish (www.teachernerd.com) so that it works without any user input. This makes it ideal for
// chaining together with Peter Kahrel's batch file processing script.
//
// All options and dialog boxes have been removed. Instead, the script just assumes that the user wants to relink ALL links
// for ALL of the common file types included in the original implementation. The script will only alert the user if there were
// any links on the page that could NOT be found. This allows the user to go back and relink those files manually after the batch
// operation is complete.
//
// The script can only find the missing links if they are in a folder with the same name as the InDesign file. This folder also needs to
// be located in the same directory as the InDesign file. This is of admittedly minimal utility to many users, but it is perfect
// for the use case for which this script modification was origianlly developed...
//=================================================================================================================================//

//--icon for the "folderPicker" button
var scriptArgs =[];
app.doScript(relinkFiles, ScriptLanguage.JAVASCRIPT, scriptArgs, UndoModes.ENTIRE_SCRIPT, "Relink Files");
  


//=================================================================================================================================//
//-- The main relinking function...
function relinkFiles(args){
    //-- Parse the input arguments array into variables for this function
    var filename = app.activeDocument.name.split(".")[0]
    var folderPath = app.activeDocument.filePath + "/" + filename
	var extensionsArray = ["jpg", "jpeg", "png", "tif", "tiff", "gif", "bmp", "ai", "eps", "pdf", "psd", "indd", "txt", "rtf", "doc", "docx", "xls", "xlsx"];
    var currentDoc = app.activeDocument;
    var relinkMethod = ["allLinks"];
    var linkCount_pBar = 0;
    

    //-- Get number of links to create a progress bar.
    var linkCount_pBar = linkCount_pBar + currentDoc.links.length;
    

    //-- Create the progress bar
    var pBarWin = new Window ('palette {text: "Relinking Files..."}');
    var pBar = pBarWin.add ('progressbar', undefined, 0, linkCount_pBar);
    pBar.preferredSize.width = 300;
    var pBarStatus = pBarWin.add("statictext", undefined, "Processing...");
    pBarStatus.preferredSize.width = 300;
    pBarStatus.alignment = 'left';
    pBarWin.show();
    
    //define tracking variables for the message displayed at the script's end.
    var missingCount = 0;
    var modifiedCount = 0;
    var notFoundCount = 0;
    var totalLinkCount = 0;

	// Get the document links
	var docLinks = currentDoc.links;
	var totalLinkCount = totalLinkCount + docLinks.length;
	
	// Open a file to record the output results
	var date = new Date();
	var reportFile = new File("TN_Relinker_Errors_" + date.getFullYear()
								+ "-" + date.getMonth() + "-" + date.getDate()
								+ "_" + date.getHours() + date.getMinutes() + date.getSeconds() + ".txt");
    reportFile.open("a");
    	
	// Perform the relinking operation
	for(aLink=0;aLink<docLinks.length;aLink++){
		var currentLink = docLinks[aLink];
		
		//-- If the currentLink is a modified link...
		if(currentLink.status == LinkStatus.linkOutOfDate){
			var modifiedCount = modifiedCount + 1;
			currentLink.update();
		}
		//--Update the progress bar
		pBarStatus.text = "Processing link " + (Number(aLink) + 1) + " of " + docLinks.length + " in " + app.activeDocument.name;
		
	   //-- If the currentLink is a missing link... 
	   if(currentLink.status == LinkStatus.LINK_MISSING ){
			var missingCount = missingCount + 1;			
			var imgFolder = Folder(folderPath).getFiles(currentLink.name+".*")
			try{
				currentLink.relink(imgFolder[0]);
			}catch(e){
				var notFoundCount = notFoundCount + 1;
			}
		}//end check link status
	pBar.value = aLink + 1;
	}//end link loop
	//-- update the progress bar


    //-- Close the progress bar
    pBarWin.close();
 
    //-- Display merror essage if any links are not found... 
    if(notFoundCount > 0){
		reportFile.write(app.activeDocument.name + "\r");
    }
	reportFile.close();
	
}//end main relinking function
//=================================================================================================================================//

function GetFileName(fileName){
    var str = "";
    var res = fileName.lastIndexOf(".");
    if (res == -1) {str = fileName;}else {str = fileName.substr(0, res);}
    return str;
}

